package jp.jasminesoft.wagby.tests.t001_Customer;

import static com.codeborne.selenide.CollectionCondition.*;
import static com.codeborne.selenide.Condition.*;
import static com.codeborne.selenide.Selenide.*;

import static jp.jasminesoft.jfc.test.support.selenide.Operations.*;
import static jp.jasminesoft.util.ExcelFunction.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.Date;

import com.codeborne.selenide.Configuration;
import com.codeborne.selenide.WebDriverRunner;

import jp.jasminesoft.jfc.test.support.selenide.ComboBox;
import jp.jasminesoft.jfc.test.support.selenide.DatePicker;
import jp.jasminesoft.jfc.test.support.selenide.DateTextBox;
import jp.jasminesoft.jfc.test.support.selenide.ListBox;
import jp.jasminesoft.jfc.test.support.selenide.RadioButton;
import jp.jasminesoft.jfc.test.support.selenide.WebContainerModelitem;
import jp.jasminesoft.jfc.test.support.selenide.WebModel;
import jp.jasminesoft.jfc.test.support.selenide.WebModelitem;
import jp.jasminesoft.jfc.test.support.selenide.WebMultiModelitem;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

/**
 * 1. Customer モデルのテスト
 *
 * @author JasmineSoft
 * @version $Revision$ $Date$
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class CustomerTest {

    /**
     * テストの前処理を行います。
     */
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        // アクセスする URL のベース
        Configuration.baseUrl = "http://localhost:8921/wagby";
        // 利用するブラウザ
        Configuration.browser = WebDriverRunner.CHROME;
        // Selenium(WebDriver) 用ドライバファイル
        System.setProperty("webdriver.chrome.driver",
                "customize/chromedriver.exe");
    }

    /**
     * 各テスト実行後の後処理を行います。
     */
    @AfterClass
    public static void tearDownAfterClass() throws Exception {
    }

    /**
     * ログオン処理のテスト
     */
    @Test
    public void test01ログオンのテスト() {
        // ログオン処理を行う。
        //Operations.logon("admin", "wagby");
        logon("admin", "wagby");
        // ページタイトルを確認
        pageTitle().shouldHave(exactText("メニュー"));
    }

    /**
     * ログオフ処理のテスト
     */
    @Test
    public void test02ログオフのテスト() {
        // ログオフ処理を行う。
        logoff();
        // checkNoErrors(); // 不要。
        // HTMLのタイトルを確認
        assertThat(title(), is("Wagby アプリケーション ログオン"));
    }

    /**
     * メニューのテスト
     */
    @Test
    public void test04メニューのテスト() {
        // ログオン処理を行う。
        logon("admin", "wagby");

        // ページタイトルを確認
        pageTitle().shouldHave(exactText("メニュー"));

        // サブメニューでの画面遷移
        // 大項目「サービス」を選択後、プルダウンから「顧客検索」を選択する
        selectMenu("サービス", "顧客情報検索");
        pageTitle().shouldHave(exactText("顧客情報 検索"));

        // メニュー画面に戻る。
        menu();
        pageTitle().shouldHave(exactText("メニュー"));

        // 大項目「サービス」を選択後、プルダウンから「顧客情報検索」を選択する
        selectSubMenu("サービス", "顧客情報検索");
        pageTitle().shouldHave(exactText("顧客情報 検索"));
    }

    /**
     * 登録画面への遷移のテスト
     */
    @Test
    public void test05登録画面への遷移テスト() {
        // 「登録画面へ」ボタンをクリック
        //clickNewButton("customer");

        // 登録画面用ボタンが一つの場合は"customer"は省略可
        clickNewButton();

        //clickEditButton(); //更新用(ルールは「登録画面へ」のボタンと同じ)

        pageTitle().shouldHave(exactText("顧客情報 新規登録"));
    }

    /**
     * 保存処理のテスト
     */
    @Test
    public void test06保存処理のテスト() {
        // 保存させるために必須項目に値を入力。
        new WebModelitem<>(new WebModel("customer"), "name").val("a");

        // 「保存」ボタンをクリック
        save();

        // ページタイトルを確認
        pageTitle().shouldHave(exactText("顧客情報 詳細表示"));
    }

    /**
     * キャンセル処理のテスト
     */
    @Test
    public void test07キャンセル処理のテスト() {
        // まず、登録画面へ遷移する。
        clickNewButton();
        pageTitle().shouldHave(exactText("顧客情報 新規登録"));

        // 「キャンセル」ボタンをクリック
        // 「キャンセル」ボタンクリック後に表示される確認ダイアログも
        // 自動的にOKをクリックします。
        cancel();
        // ページタイトルを確認
        pageTitle().shouldHave(exactText("顧客情報 詳細表示"));
    }

    /**
     * 登録画面での各項目への入力テスト
     */
    @Test
    public void test08登録画面での各項目への入力テスト() {
        // 登録画面へ遷移する。
        clickNewButton();
        pageTitle().shouldHave(exactText("顧客情報 新規登録"));

        // モデルの情報を保持するインスタンス
        WebModel model = new WebModel("customer");
        // customerモデルのname項目への入力
        new WebModelitem<>(model, "name").val("ジャスミン太郎");

        // 「ジャスミン太郎」と入力されていることを確認
        new WebModelitem<>(model, "name").shouldHave("ジャスミン太郎");

        // ラジオボタン:「地方公共団体」を選択する
        new RadioButton(model, "customertype").val("地方公共団体");

        // 繰返し項目
        // 繰り返し項目「email」を操作するオブジェクトを作成。
        new WebMultiModelitem<>(model, "email")
                //.clear() // 既存の入力値全てを削除
                .get(1) // 1番目の入力フィールドを取得
                .val("taro@jasminesoft.co.jp")
                .add()  // 「追加」ボタンをクリック
                .val("sales@jasminesoft.co.jp");

        // こちらでも可(内部処理は同じ)
        new WebMultiModelitem<>(model, "email")
                .val("taro@jasminesoft.co.jp", "sales@jasminesoft.co.jp");

        // 繰り返しコンテナを操作するオブジェクトを作成。
        WebContainerModelitem<?> report
                = new WebContainerModelitem<>(model, "report");
        // コンテナの1行目があればこれを取得する(なければエラー)。
        WebContainerModelitem<?> report01 = report.get(1);

        // コンテナ1行目の「rnote」項目への入力
        new WebModelitem<>(report01, "rnote").val("Wagby 購入");
        DatePicker<?> datePicker
                = new DateTextBox(report01, "rdate").datePicker();
        datePicker.nextMonth();   // 翌月へ
        datePicker.prevMonth();   // 前月へ
        datePicker.nextYear();    // 翌年へ
        datePicker.prevYear();    // 前年へ
        datePicker.selectDate(1); // 1日を選択

        // コンテナの2行目がない状態で実行してもエラーとなる。
        //WebContainerModelitem report02 = report.get(2);
        // コンテナの「追加」ボタンをクリックして、新規行を取得。
        WebContainerModelitem<?> report02 = report.add();

        datePicker = new DateTextBox(report02, "rdate").datePicker();
        datePicker.nextMonth(); // 翌月へ
        datePicker.prevMonth(); // 前月へ
        datePicker.nextYear(); // 翌年へ
        datePicker.prevYear(); // 前年へ
        datePicker.selectDate(2); // 1日を選択

        // コンテナ2行目の「rnote」項目への入力
        new WebModelitem<>(report02, "rnote").val("Wagby 保守契約更新");

        // 追記型リストボックス
        ComboBox companyname = new ComboBox(model, "companyname");
        // 直接入力を行う場合(存在しない選択肢も可)
        companyname.val("ジャスミンソフト");

        // ドロップダウンから選択する場合
        // 存在しない選択肢を選んだ場合はエラー
        //companyname
        //    .dropdown()
        //    .select("ジャスミンソフト");

        // 読込専用項目の確認:最終更新者
        new WebModelitem<>(model, "updateuserid")
                .shouldBeReadOnly()
                .shouldHave("admin");

        // 読込専用項目の確認:データ作成日
        new WebModelitem<>(model, "insertrec")
                .shouldBeReadOnly()
                .shouldHave(text(TEXT(NOW(), "yyyy-MM-dd HH:")));

        // 読込専用項目の確認:データ更新日
        new WebModelitem<>(model, "updaterec")
                .shouldBeReadOnly()
                .shouldHave(text(TEXT(NOW(), "yyyy-MM-dd HH:")));

        // 「保存」ボタンをクリック
        save();

        // ページタイトルを確認
        pageTitle().shouldHave(exactText("顧客情報 詳細表示"));
    }

    /**
     * 更新画面での各項目への入力テスト
     */
    @Test
    public void test09更新画面での各項目への入力テスト() {
        // 更新画面へ遷移する。
        clickEditButton();
        pageTitle().shouldHave(exactText("顧客情報 更新"));

        // モデルの情報を保持するインスタンス
        WebModel model = new WebModel("customer");
        // customerモデルのkananame項目への入力
        new WebModelitem<>(model, "kananame").val("ジャスミンタロウ");
        new WebModelitem<>(model, "companykananame").val("ジャスミンソフト");
        new WebModelitem<>(model, "deptname").val("技術開発部");
        new WebModelitem<>(model, "title").val("一般");
        new WebModelitem<>(model, "zipcode").val("9012227");
        new WebModelitem<>(model, "address").val("沖縄県宜野湾市宇地泊902-1");
        new WebModelitem<>(model, "tel").val("098-890-6036");
        new WebModelitem<>(model, "fax").val("098-890-6038");

        // 繰り返しコンテナを操作するオブジェクトを作成。
        WebContainerModelitem<?> report
                = new WebContainerModelitem<>(model, "report");
        new ListBox(report.get(1), "rstatus").val("A");
        new ListBox(report.get(2), "rstatus").val("A");

        // 「保存」ボタンをクリック
        save();

        // ページタイトルを確認
        pageTitle().shouldHave(exactText("顧客情報 詳細表示"));
    }

    /**
     * 詳細画面での各項目の値の確認テスト
     */
    @Test
    public void test10詳細画面での各項目の入力値の確認テスト() {
        // ページタイトルを確認
        pageTitle().shouldHave(exactText("顧客情報 詳細表示"));

        // モデルの情報を保持するインスタンス
        WebModel model = new WebModel("customer");
        // customerモデルのname項目の値の確認
        new WebModelitem<>(model, "name").shouldHave("ジャスミン太郎");

        // ラジオボタン
        new RadioButton(model, "customertype").shouldHave("地方公共団体");

        // 繰返し項目
        new WebMultiModelitem<>(model, "email").shouldHave(
                "taro@jasminesoft.co.jp", "sales@jasminesoft.co.jp");

        // 繰り返しコンテナを操作するオブジェクトを作成。
        WebContainerModelitem<?> report
                = new WebContainerModelitem<>(model, "report");
        // コンテナの1行目を取得。
        WebContainerModelitem<?> report01 = report.get(1);

        Date bomonth = BOMONTH(NOW());
        String dateText = TEXT(bomonth, "yyyy-MM-dd");
        // コンテナ1行目の「rnote」項目への入力
        new WebModelitem<>(report01, "rnote").shouldHave("Wagby 購入");
        new DateTextBox(report01, "rdate").shouldHave(dateText);

        // コンテナの2行目を取得。
        WebContainerModelitem<?> report02 = report.get(2);

        dateText = TEXT(MOVEDAY(bomonth, 1), "yyyy-MM-dd");
        new DateTextBox(report02, "rdate").shouldHave(dateText);
        new WebModelitem<>(report02, "rdate").shouldHave(dateText);

        // コンテナ2行目の「rnote」項目の値の確認
        new WebModelitem<>(report02, "rnote").shouldHave("Wagby 保守契約更新");

        // 追記型リストボックス
        new ComboBox(model, "companyname").shouldHave("ジャスミンソフト");

        // 読込専用項目の確認:最終更新者
        new WebModelitem<>(model, "updateuserid")
                .shouldHave("admin");

        // 読込専用項目の確認:データ作成日
        new WebModelitem<>(model, "insertrec")
                .shouldHave(text(TEXT(NOW(), "yyyy-MM-dd HH:")));

        // 読込専用項目の確認:データ更新日
        new WebModelitem<>(model, "updaterec")
                .shouldHave(text(TEXT(NOW(), "yyyy-MM-dd HH:")));
    }

    /**
     * 詳細画面での各項目の値の確認テスト2
     */
    @Test
    public void test11詳細画面での各項目の入力値の確認テスト2() {
        // ページタイトルを確認
        pageTitle().shouldHave(exactText("顧客情報 詳細表示"));

        // モデルの情報を保持するインスタンス
        WebModel model = new WebModel("customer");
        // customerモデルのkananame項目への入力
        new WebModelitem<>(model, "kananame").shouldHave("ジャスミンタロウ");
        new WebModelitem<>(model, "companykananame").shouldHave("ジャスミンソフト");
        new WebModelitem<>(model, "deptname").shouldHave("技術開発部");
        new WebModelitem<>(model, "title").shouldHave("一般");
        new WebModelitem<>(model, "zipcode").shouldHave("9012227");
        new WebModelitem<>(model, "address").shouldHave("沖縄県宜野湾市宇地泊902-1");
        new WebModelitem<>(model, "tel").shouldHave("098-890-6036");
        new WebModelitem<>(model, "fax").shouldHave("098-890-6038");

        // 繰り返しコンテナを操作するオブジェクトを作成。
        WebContainerModelitem<?> report
                = new WebContainerModelitem<>(model, "report");
        new ListBox(report.get(1), "rstatus").shouldHave("A");
        new ListBox(report.get(2), "rstatus").shouldHave("A");
    }

    /**
     * ログオフ
     */
    @Test
    public void test99ログオフ() {
        logoff();
    }
}
